@@ -40,9 +40,12 @@ hook_plot_md_base = function(x, options) {
   plot1 = ai || options$fig.cur <= 1L
   plot2 = ai || options$fig.cur == options$fig.num
   if (is.null(w) && is.null(h) && is.null(s) && a == 'default' && !pandoc_html) {
+    # append \ to ![]() to prevent the figure environment in these cases
+    nocap = cap == '' && !is.null(pandoc_to()) && (options$fig.num == 1 || ai) &&
+      !grepl('-implicit_figures', pandoc_from())
     return(sprintf(
       '![%s](%s%s)%s%s', cap, base, .upload.url(x),
-      if (cap == '' && !is.null(pandoc_to())) '\\' else '', if (plot2) '' else ' '
+      if (nocap) '<!-- -->' else '', if (is_latex_output()) ' ' else ''
     ))
   }
   # use HTML syntax <img src=...>
